@@ -53,7 +53,7 @@ spark_config <- function(file = "config.yml", use_default = TRUE) {
     mergedConfig$master$`sparklyr.shell.driver-class-path` <- Sys.getenv("SPARK_DRIVER_CLASSPATH")
   }
 
-  if (is.null(spark_config_value(mergedConfig, c("sparklyr.connect.cores.local", "sparklyr.cores.local")))) {
+  if (is.null(spark_config_value(mergedConfig, c("sparklyr.cores.local", "sparklyr.connect.cores.local")))) {
     mergedConfig$sparklyr.connect.cores.local <- parallel::detectCores()
   }
 
@@ -176,8 +176,9 @@ spark_config_packages <- function(config, packages, version, scala_version = NUL
     config$sparklyr.shell.packages <- c(
       config$sparklyr.shell.packages,
       sprintf(
-        "io.delta:delta-core_%s:0.4.0",
-        if (version >= "3.0.0") "2.12" else scala_version %||% "2.11"
+        "io.delta:delta-core_%s:%s",
+        if (version >= "3.0.0") "2.12" else scala_version %||% "2.11",
+        if (version >= "3.1.0") "1.0.0" else if (version >= "3.0.0") "0.8.0" else "0.6.1"
       )
     )
   }
