@@ -166,7 +166,7 @@ readTypedObject <- function(con, type) {
     "l" = readList(con),
     "e" = readMap(con),
     "s" = readStruct(con),
-    "f" = readFastStringArray(con),
+    "f" = readStringArray(con),
     "n" = NULL,
     "j" = getJobj(con, readString(con)),
     "J" = jsonlite::fromJSON(
@@ -179,24 +179,35 @@ readTypedObject <- function(con, type) {
 
 readString <- function(con) {
   stringLen <- readInt(con)
-  string <- ""
 
-  if (stringLen > 0) {
-    raw <- read_bin(con, raw(), stringLen, endian = "big")
-    if (is.element("00", raw)) {
-      warning("Input contains embedded nuls, removing.")
-      raw <- raw[raw != "00"]
+  string <- (
+    if (stringLen > 0) {
+      raw <- read_bin(con, raw(), stringLen, endian = "big")
+      if (is.element("00", raw)) {
+        warning("Input contains embedded nuls, removing.")
+        raw <- raw[raw != "00"]
+      }
+      rawToChar(raw)
+    } else if (stringLen == 0) {
+      ""
+    } else {
+      NA_character_
     }
-    string <- rawToChar(raw)
-  }
+  )
 
   Encoding(string) <- "UTF-8"
   string
 }
 
-readFastStringArray <- function(con) {
+readStringArray <- function(con) {
   joined <- readString(con)
-  as.list(strsplit(joined, "\u0019")[[1]])
+  arr <- as.list(strsplit(joined, "\u0019")[[1]])
+  lapply(
+    arr,
+    function(x) {
+      if (x == "<NA>") NA_character_ else x
+    }
+  )
 }
 
 readDateArray <- function(con, n = 1) {
@@ -240,7 +251,7 @@ readDate <- function(con) {
   if (is.na(n)) {
     as.Date(NA)
   } else {
-    d <- as.Date(n, origin = "1970-01-01", tz = "UTC")
+    d <- as.Date(n, origin = "1970-01-01")
     if (getOption("sparklyr.collect.datechars", FALSE)) {
       as.character(d)
     } else {
@@ -255,7 +266,7 @@ readTime <- function(con, n = 1) {
   } else {
     t <- readDouble(con, n)
 
-    r <- as.POSIXct(t, origin = "1970-01-01", tz = "UTC")
+    r <- as.POSIXct(t, origin = "1970-01-01")
     if (getOption("sparklyr.collect.datechars", FALSE)) {
       as.character(r)
     } else {
@@ -344,7 +355,9 @@ readStruct <- function(con) {
 
 readRaw <- function(con) {
   dataLen <- readInt(con)
-  if (dataLen == 0) {
+  if (dataLen == -1) {
+    NA
+  } else if (dataLen == 0) {
     raw()
   } else {
     read_bin(con, raw(), as.integer(dataLen), endian = "big")
@@ -567,10 +580,11 @@ core_invoke_cancel_running <- function(sc) {
   if (exists("connection_progress_terminated")) connection_progress_terminated(sc)
 }
 
-write_bin_args <- function(backend, object, static, method, args) {
+write_bin_args <- function(backend, object, static, method, args, return_jobj_ref = FALSE) {
   rc <- rawConnection(raw(), "r+")
   writeString(rc, object)
   writeBoolean(rc, static)
+  writeBoolean(rc, return_jobj_ref)
   writeString(rc, method)
 
   writeInt(rc, length(args))
@@ -625,14 +639,14 @@ core_invoke_socket_name <- function(sc) {
 }
 
 core_remove_jobjs <- function(sc, ids) {
-  core_invoke_method_impl(sc, static = TRUE, noreply = TRUE, "Handler", "rm", as.list(ids))
+  core_invoke_method_impl(sc, static = TRUE, noreply = TRUE, "Handler", "rm", FALSE, as.list(ids))
 }
 
-core_invoke_method <- function(sc, static, object, method, ...) {
-  core_invoke_method_impl(sc, static, noreply = FALSE, object, method, ...)
+core_invoke_method <- function(sc, static, object, method, return_jobj_ref, ...) {
+  core_invoke_method_impl(sc, static, noreply = FALSE, object, method, return_jobj_ref, ...)
 }
 
-core_invoke_method_impl <- function(sc, static, noreply, object, method, ...) {
+core_invoke_method_impl <- function(sc, static, noreply, object, method, return_jobj_ref, ...) {
   # N.B.: the reference to `object` must be retained until after a value or exception is returned to us
   # from the invoked method here (i.e., cannot have `object <- something_else` before that), because any
   # re-assignment could cause the last reference to `object` to be destroyed and the underlying JVM object
@@ -679,7 +693,7 @@ core_invoke_method_impl <- function(sc, static, noreply, object, method, ...) {
   # if the object is a jobj then get it's id
   objId <- ifelse(inherits(object, "spark_jobj"), object$id, object)
 
-  write_bin_args(backend, objId, static, method, args)
+  write_bin_args(backend, objId, static, method, args, return_jobj_ref)
 
   if (identical(object, "Handler") &&
     (identical(method, "terminateBackend") || identical(method, "stopBackend"))) {
@@ -1031,7 +1045,7 @@ writeObject <- function(con, object, writeType = TRUE) {
   type <- class(object)[[1]]
 
   if (type %in% c("integer", "character", "logical", "double", "numeric", "factor", "Date", "POSIXct")) {
-    if (is.na(object)) {
+    if (is.na(object) && !is.nan(object)) {
       object <- NULL
       type <- "NULL"
     }
@@ -1281,6 +1295,7 @@ spark_worker_init_packages <- function(sc, context) {
 }
 
 spark_worker_execute_closure <- function(
+                                         sc,
                                          closure,
                                          df,
                                          funcContext,
@@ -1299,7 +1314,6 @@ spark_worker_execute_closure <- function(
     worker_log("found barrier execution context")
     barrier_arg <- list(barrier = barrier_map)
   }
-
   closure_params <- length(formals(closure))
   has_partition_index_param <- (
     !is.null(funcContext$partition_index_param) &&
@@ -1325,7 +1339,8 @@ spark_worker_execute_closure <- function(
   options(stringsAsFactors = FALSE)
 
   if (identical(fetch_result_as_sdf, FALSE)) {
-    result <- lapply(result, function(x) serialize(x, NULL))
+    serialize_impl <- spark_worker_get_serializer(sc)
+    result <- lapply(result, function(x) serialize_impl(x, NULL))
     class(result) <- c("spark_apply_binary_result", class(result))
     result <- tibble::tibble(spark_apply_binary_result = result)
   }
@@ -1403,9 +1418,16 @@ spark_worker_apply_maybe_schema <- function(config, result) {
 spark_worker_build_types <- function(context, columns) {
   names <- names(columns)
   sqlutils <- worker_invoke(context, "getSqlUtils")
-  fields <- lapply(names, function(name) {
-    worker_invoke(sqlutils, "createStructField", name, columns[[name]][[1]], TRUE)
-  })
+  fields <- worker_invoke(
+    sqlutils,
+    "createStructFields",
+    lapply(
+      names,
+      function(name) {
+        list(name, columns[[name]][[1]], TRUE)
+      }
+    )
+  )
 
   worker_invoke(sqlutils, "createStructType", fields)
 }
@@ -1435,14 +1457,31 @@ spark_worker_add_group_by_column <- function(df, result, grouped, grouped_by) {
   result
 }
 
+get_arrow_converters <- function(context, config) {
+  if (config$spark_version < "2.3.0") {
+    stop("ArrowConverters is only supported for Spark 2.3 or above.")
+  }
+
+  worker_invoke(context, "getArrowConverters")
+}
+
+get_arrow_converters_impl <- function(context, config) {
+  if (config$spark_version < "2.3.0") {
+    stop("ArrowConverters is only supported for Spark 2.3 or above.")
+  }
+
+  worker_invoke(context, "getArrowConvertersImpl")
+}
+
 spark_worker_apply_arrow <- function(sc, config) {
   worker_log("using arrow serializer")
 
   context <- spark_worker_context(sc)
   spark_worker_init_packages(sc, context)
 
-  closure <- unserialize(worker_invoke(context, "getClosure"))
-  funcContext <- unserialize(worker_invoke(context, "getContext"))
+  deserialize_impl <- spark_worker_get_deserializer(sc)
+  closure <- deserialize_impl(worker_invoke(context, "getClosure"))
+  funcContext <- deserialize_impl(worker_invoke(context, "getContext"))
   grouped_by <- worker_invoke(context, "getGroupBy")
   grouped <- !is.null(grouped_by) && length(grouped_by) > 0
   columnNames <- worker_invoke(context, "getColumns")
@@ -1458,9 +1497,9 @@ spark_worker_apply_arrow <- function(sc, config) {
     record_batch_raw <- spark_worker_get_group_batch(record_batch_raw_groups[[record_batch_raw_groups_idx]])
   } else {
     row_iterator <- worker_invoke(context, "getIterator")
-    arrow_converter_impl <- worker_invoke(context, "getArrowConvertersImpl")
+    arrow_converters_impl <- get_arrow_converters_impl(context, config)
     record_batch_raw <- worker_invoke(
-      arrow_converter_impl,
+      arrow_converters_impl,
       "toBatchArray",
       row_iterator,
       schema_input,
@@ -1489,6 +1528,7 @@ spark_worker_apply_arrow <- function(sc, config) {
       colnames(df) <- columnNames[seq_along(colnames(df))]
 
       result <- spark_worker_execute_closure(
+        sc,
         closure,
         df,
         funcContext,
@@ -1531,8 +1571,8 @@ spark_worker_apply_arrow <- function(sc, config) {
   if (length(all_batches) > 0) {
     worker_log("updating ", total_rows, " rows using ", length(all_batches), " row batches")
 
-    arrow_converter <- worker_invoke(context, "getArrowConverters")
-    row_iter <- worker_invoke(arrow_converter, "fromPayloadArray", all_batches, schema_output)
+    arrow_converters <- get_arrow_converters(context, config)
+    row_iter <- worker_invoke(arrow_converters, "fromPayloadArray", all_batches, schema_output)
 
     worker_invoke(context, "setResultIter", row_iter)
     worker_log("updated ", total_rows, " rows using ", length(all_batches), " row batches")
@@ -1543,6 +1583,19 @@ spark_worker_apply_arrow <- function(sc, config) {
   worker_log("finished apply")
 }
 
+spark_worker_get_serializer <- function(sc) {
+  serializer <- unserialize(worker_invoke(spark_worker_context(sc), "getSerializer"))
+  if (is.list(serializer)) {
+    function(x, ...) serializer$serializer(x)
+  } else {
+    serializer
+  }
+}
+
+spark_worker_get_deserializer <- function(sc) {
+  unserialize(worker_invoke(spark_worker_context(sc), "getDeserializer"))
+}
+
 spark_worker_apply <- function(sc, config) {
   context <- spark_worker_context(sc)
   spark_worker_init_packages(sc, context)
@@ -1557,11 +1610,13 @@ spark_worker_apply <- function(sc, config) {
   groups <- worker_invoke(context, if (grouped) "getSourceArrayGroupedSeq" else "getSourceArraySeq")
   worker_log("retrieved ", length(groups), " rows")
 
+  deserialize_impl <- spark_worker_get_deserializer(sc)
+
   closureRaw <- worker_invoke(context, "getClosure")
-  closure <- unserialize(closureRaw)
+  closure <- deserialize_impl(closureRaw)
 
   funcContextRaw <- worker_invoke(context, "getContext")
-  funcContext <- unserialize(funcContextRaw)
+  funcContext <- deserialize_impl(funcContextRaw)
 
   closureRLangRaw <- worker_invoke(context, "getClosureRLang")
   if (length(closureRLangRaw) > 0) {
@@ -1593,7 +1648,22 @@ spark_worker_apply <- function(sc, config) {
       if (config$single_binary_column) {
         tibble::tibble(encoded = lapply(data, function(x) x[[1]]))
       } else {
-        do.call(rbind.data.frame, c(data, list(stringsAsFactors = FALSE)))
+        bind_rows <- core_get_package_function("dplyr", "bind_rows")
+        as_tibble <- core_get_package_function("tibble", "as_tibble")
+        if (!is.null(bind_rows) && !is.null(as_tibble)) {
+          do.call(
+            bind_rows,
+            lapply(
+              data, function(x) { as_tibble(x, .name_repair = "universal") }
+            )
+          )
+        } else {
+          warning("dplyr::bind_rows or tibble::as_tibble is unavailable, ",
+                  "falling back to rbind implementation in base R. ",
+                  "Inputs with list column(s) will not work.")
+
+          do.call(rbind.data.frame, c(data, list(stringsAsFactors = FALSE)))
+        }
       })
 
     if (!config$single_binary_column) {
@@ -1624,6 +1694,7 @@ spark_worker_apply <- function(sc, config) {
     colnames(df) <- columnNames[seq_along(colnames(df))]
 
     result <- spark_worker_execute_closure(
+      sc,
       closure,
       df,
       funcContext,
@@ -1801,7 +1872,7 @@ worker_connection.spark_jobj <- function(x, ...) {
 # nocov start
 
 worker_invoke_method <- function(sc, static, object, method, ...) {
-  core_invoke_method(sc, static, object, method, ...)
+  core_invoke_method(sc, static, object, method, FALSE, ...)
 }
 
 worker_invoke <- function(jobj, method, ...) {
